/*****************************************************************************
 ***      Author:  Robbie Gubler -- tarsin@happybox.org                    ***
 ***        Date:  2001.04.28                                              ***
 *** Description:  TEA(Tiny Encryption Algorithm) Command line Interface.  ***
 ***    Comments:  The TEA is a Feistel type cipher; the block is split    ***
 ***               into two parts, each being 32 bits, 128 bit key, and by ***
 ***               default uses 32 rounds.  The cipher was orginally       ***
 ***               written by David Wheeler and later modifed by Roger M.  ***
 ***               Needham.                                                ***
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curses.h>
#include "TEA-ICommandLine.h"
#include "TEA.h"

#define TRUE    1
#define FALSE   -1
#define REVISION         "2001.05.18.r02"
#define DELIMITER       '-'
#define DECIPHER        'd'
#define ENCIPHER        'e'
#define KEY		'k'
#define LINE            'l'
#define HELP            'h'


int main(int argc, char* argv[]) {
    int i, n; 
    int err = 0;
    int enabled_flags[] = {0,0,0,0,0};
    unsigned long key[4];

    if(argc <= 1) 
        return PrintInfo(argc, argv);   

    for(i = 1, n = 0; argv[i] != NULL; i++) {   /* validate flags */
        if(argv[i][n] == DELIMITER) {
            n++;
            while(argv[i][n] != '\0') {
                switch(argv[i][n]) {
                case ENCIPHER:
                    if(FirstTime(0))
                        enabled_flags[0] = i+1;
                    else
                        err = 1;
                break;
                case DECIPHER:
                     if(FirstTime(0)) {
                         enabled_flags[1] = i+1;
                     } else
                        err = 1;
                break;
                case KEY:
                     if(FirstTime(2)) {
                         enabled_flags[2] = i+1;
                     } else
                        err = 1;
                break;
                case LINE:
                     if(FirstTime(3)) {
                         enabled_flags[3] = i+1;
                     } else
                        err = 1;
                break;
                case HELP:
                     if(FirstTime(4)) {
                         enabled_flags[4] = i+1;
                     } else
                        err = 1;
                break;

                default:
                    err = 1;
                }
                n++;
            }
            n = 0;
        } else if(argv[i-1][0] != DELIMITER)  
            return PrintInfo(argc, argv);
    }
    if(err == 1) 
        return PrintInfo(argc, argv);
    if(enabled_flags[4])    /* print extended info */
        return PrintExtendedInfo(argc, argv);
    if(enabled_flags[2]) {    /* get key from file.  key crunch method*/
        unsigned long ln, sz;

        if(enabled_flags[3]/*line,size*/) {
            char* buffer = argv[(enabled_flags[3])];
            char tmp[6] = {0};
            int i, virgin = 1;

            for(i = 0; buffer[i]; i++) {
                if(buffer[i] != ',')
                    strncat(tmp, &buffer[i], 1);
                else {
                    if(virgin) {
                        ln = atoi(tmp);
                        virgin = 0;
                        memset(tmp, 0, 6);
                    }
                }
            }
            sz = atoi(tmp);

        } else {
            ln = 1;
            sz = 128;
        }
        
        if(GetPassPhraseFromFile(key, argv[(enabled_flags[2])], ln, sz) > 0)
            return 1;

    } else if(ftell(stdin) <= 0) { /* get key from piped stdin. key crunch method*/
        unsigned char* buffer;
        unsigned char fi_size;
        
        fseek(stdin, 1, SEEK_END);
        fi_size = (ftell(stdin)-1);
        rewind(stdin);
        
        buffer = calloc(fi_size+1, 1);
        fread(buffer, fi_size, 1, stdin);
        if(buffer[strlen(buffer)-2] == ' ')
            buffer[strlen(buffer)-2] = '\0';
        else
            buffer[strlen(buffer)-1] = '\0';
        
        if(KeyCrunch(key, buffer) == TRUE) {
            printf("\nError: Pass Phrase is less than 8 bytes.\n");
            return 1;
        } 

        printf("\nUsing %d bytes for pass phrase from STDIN.",strlen(buffer));
        printf("\nPass phrase = \"%s\"", buffer);

    } else {    /* get key durring run time */
        double x;
        char* stopstr;
        char str[7];
        initscr();
        cbreak();
        printw("\nEnter the key in hex notation:");
	for(i = 0; i < 4; i++) {
            printw("\n\tKey[%d] = 0x", i);
	    refresh();
            __getstr(str, 7);
            x = strtod(str, &stopstr);
            key[i] = strtol(str, &stopstr, 16);
        }
	endwin();
  }
 
    printf("\n\nKey[0] = 0x%x\nKey[1] = 0x%x\nKey[2] = 0x%x\nKey[3] = 0x%x", key[0], key[1], key[2], key[3]);

    if(enabled_flags[0]) {    /* encipher file */
        ReadFile(ENCIPHER, key, argv[(enabled_flags[0])]);
    } else if (enabled_flags[1]) {    /* decipher file */
        ReadFile(DECIPHER, key, argv[(enabled_flags[1])]);
    }

    printf("\n\n");
    return 0;
}

int FirstTime(int n) {
   short temp[32] = {0};
   

    if(n <= 31) {
        if(temp[n] != 1) 
            temp[n] = 1;
        else
            return 0;
    } else if(n > 31)
        return 0;

    return 1;

}
int PrintInfo(int argc, char* argv[]) {
    printf("\nTEA(Tiny Encryption Algorithm)");
    printf("\n  Implementation by: Robbie Gubler -- tarsin@happybox.org");
    printf("\n            Revision: %s", REVISION);
    printf("\n  Flags:");
    printf("\n    %c%c  [filename] - Encipher a plaintext file.", DELIMITER, ENCIPHER);
    printf("\n    %c%c  [filename] - Decipher a ciphertext file.", DELIMITER, DECIPHER);
    printf("\n    %c%c  [filename] - Extract the key from a file. (optional)", DELIMITER, KEY);
    printf("\n    %c%c  [ln],[sz]  - Get pass phrase at line [ln] and read [sz] bytes.", DELIMITER, LINE);
    printf("\n    %c%c             - Help and other information.", DELIMITER, HELP);
    printf("\n\n");

    return -1;
}  

int PrintExtendedInfo(int argc, char* argv[]) {
    printf("\nTEA(Tiny Encryption Algorithm)");
    printf("\n  Implementation by: Robbie Gubler -- tarsin@happybox.org");
    printf("\n            Revision: %s", REVISION);
    printf("\n\n\n  Flags:");
    printf("\n    %c%c  [filename] - Encipher a plaintext file. Upon completion of the", DELIMITER, ENCIPHER);
    printf("\n                       encipherment the ciphertext is written to"); 
    printf("\n                       [filename].tea-enc.");
    printf("\n    %c%c  [filename] - Decipher a ciphertext file. Upon completion of the", DELIMITER, DECIPHER);
    printf("\n                       decipherment the plaintext is written to ");
    printf("\n                       [filename].tea-dec.");
    printf("\n    %c%c  [filename] - The file to extract the pass phrase from.  The pass", DELIMITER, KEY);
    printf("\n                       phrase is then put through a key crunching routine");
    printf("\n                       to generate the key used for the encipherment.");
    printf("\n    %c%c  [ln],[sz]  - Specifys the starting line and desired pass phrase", DELIMITER, LINE);
    printf("\n                       size(length).  This parameter requires that '%c%c'", DELIMITER, KEY);
    printf("\n                       is also used.  However If this parameter is not");
    printf("\n                       specified then by default the function will extract");
    printf("\n                       a 128 byte pass phrase at line 1.");
    printf("\n    %c%c             - Help and other information; this screen.", DELIMITER, HELP);
    printf("\n\n\n  Interface Info:");
    printf("\n    Upon completion of the encipherment the ciphertext is written");
    printf("\n    to [filename].tea-enc.  Inversly the deciphered text is");
    printf("\n    written to [filename].tea-dec.");
    printf("\n\n    When the '%c%c' argument is used without '%c%c' by default", DELIMITER, KEY, DELIMITER, LINE);
    printf("\n    the first 128 bytes of [filename] will used as the pass phrase.");
    printf("\n\n    The pass phrase is divided into 4 parts, each part is hashed");
    printf("\n    and used as portion of the key.  Generally you're probably going");
    printf("\n    to want to utilize the pass phrase option, but the key can be");
    printf("\n    specified in hex notation durring run-time as well.");
    printf("\n\n\n  Syntax Examples:");
    printf("\n      %s -e document.txt -k phrase.txt", argv[0]);
    printf("\n      %s -e document.txt -k docs/Walden.doc -l300,256", argv[0]);
    printf("\n      %s -d document.txt.tea-enc", argv[0]);
    printf("\n      echo \"this is my pass phrase\" | %s -d text.txt.tea-enc", argv[0]);
    printf("\n\n\n  Cipher Info:");
    printf("\n        The TEA is a Feistel type cipher; the block is split");
    printf("\n        into two parts, each being 32 bits, 128 bit key, and by");
    printf("\n        default uses 32 rounds.  The cipher was orginally");
    printf("\n        written by David Wheeler and later modifed by Roger M.");
    printf("\n        Needham.");

    printf("\n\n");

    return -1;
}

void StdOutDisplay(char* str) {
    if(str == NULL || !(strcmp(str, "")))
        return;
    
    printf("\n%s", str);

}

char* __getstr(char* str, int len) {
    char ch;
    int discard = 0;
    int i;
    
    memset(str, 0, len+1);
    fflush(stdin);

    for(i = 0; i < len && !discard;) {
        ch = getch();
        if(ch != '\n' && ch != '\0' && ch != '\f' && ch != '\r' && ch != '\b') {
            str[i] = ch;
            i++;
        } else if(ch != '\b')
            discard = TRUE;
    }

    return str;
}
